@@ -12,18 +12,18 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import itertools
 import logging
 import threading
 from concurrent.futures.thread import ThreadPoolExecutor
-from operator import attrgetter
 
-from collections import Counter
-from collections import namedtuple
+from collections import Counter, defaultdict, namedtuple
 
 from nodepool import exceptions
 from nodepool import nodeutils
 from nodepool import zk
 from nodepool.driver import Provider
+from nodepool.driver.utils import NodeDeleter
 from nodepool.driver.static.handler import StaticNodeRequestHandler
 
 
@@ -101,17 +101,29 @@ class StaticNodeProvider(Provider):
         return nodes
 
     def getWaitingNodesOfType(self, labels):
-        nodes = []
+        """Get all waiting nodes of a type.
+
+        Nodes are sorted in ascending order by the associated request's
+        priority, which means that they are in descending order of the
+        priority value (a lower value means the request has a higher
+        priority).
+        """
+        nodes_by_prio = defaultdict(list)
         for node in self.zk.nodeIterator():
             if (node.provider != self.provider.name or
                 node.state != zk.BUILDING or
-                not set(node.type).issubset(labels)
+                not set(node.type).issubset(labels) or
+                not node.allocated_to
             ):
                 continue
-            nodes.append(node)
-        return list(
-            sorted(nodes, key=attrgetter("created_time"), reverse=True)
-        )
+            request = self.zk.getNodeRequest(node.allocated_to, cached=True)
+            if request is None:
+                continue
+            nodes_by_prio[request.priority].append(node)
+
+        return list(itertools.chain.from_iterable(
+            nodes_by_prio[p] for p in sorted(nodes_by_prio, reverse=True)
+        ))
 
     def checkNodeLiveness(self, node):
         node_tuple = nodeTuple(node)
@@ -187,12 +199,14 @@ class StaticNodeProvider(Provider):
             node.pool = pool_name
             node.launcher = "static driver"
             node.type = static_node["labels"]
+            node.external_id = static_node["name"]
             node.hostname = static_node["name"]
             node.username = static_node["username"]
             node.interface_ip = static_node["name"]
             node.connection_port = static_node["connection-port"]
             node.connection_type = static_node["connection-type"]
             node.python_path = static_node["python-path"]
+            node.shell_type = static_node["shell-type"]
             nodeutils.set_node_ip(node)
             node.host_keys = host_keys
             node.attributes = pool.node_attributes
@@ -217,14 +231,15 @@ class StaticNodeProvider(Provider):
             static_node["username"],
             static_node["connection-port"],
             static_node["connection-type"],
+            static_node["shell-type"],
             static_node["python-path"],
             host_keys,
         )
 
         for node in nodes:
             original_attrs = (node.type, node.username, node.connection_port,
-                              node.connection_type, node.python_path,
-                              node.host_keys)
+                              node.shell_type, node.connection_type,
+                              node.python_path, node.host_keys)
 
             if original_attrs == new_attrs:
                 continue
@@ -235,6 +250,7 @@ class StaticNodeProvider(Provider):
                 node.username = static_node["username"]
                 node.connection_port = static_node["connection-port"]
                 node.connection_type = static_node["connection-type"]
+                node.shell_type = static_node["shell-type"]
                 node.python_path = static_node["python-path"]
                 nodeutils.set_node_ip(node)
                 node.host_keys = host_keys
@@ -385,6 +401,11 @@ class StaticNodeProvider(Provider):
             for n in p.nodes
         }
 
+    def startNodeCleanup(self, node):
+        t = NodeDeleter(self.zk, self, node)
+        t.start()
+        return t
+
     def cleanupNode(self, server_id):
         return True
 
