@@ -109,6 +109,73 @@ class NodeLauncher(threading.Thread,
             self.log.exception("Exception while reporting stats:")
 
 
+class NodeDeleter(threading.Thread):
+    log = logging.getLogger("nodepool.NodeDeleter")
+
+    def __init__(self, zk, provider_manager, node):
+        threading.Thread.__init__(self, name='NodeDeleter for %s %s' %
+                                  (node.provider, node.external_id))
+        self._zk = zk
+        self._provider_manager = provider_manager
+        self._node = node
+
+    @staticmethod
+    def delete(zk_conn, manager, node, node_exists=True):
+        '''
+        Delete a server instance and ZooKeeper node.
+
+        This is a class method so we can support instantaneous deletes.
+
+        :param ZooKeeper zk_conn: A ZooKeeper object to use.
+        :param ProviderManager provider_manager: ProviderManager object to
+            use fo deleting the server.
+        :param Node node: A locked Node object that describes the server to
+            delete.
+        :param bool node_exists: True if the node actually exists in ZooKeeper.
+            An artifical Node object can be passed that can be used to delete
+            a leaked instance.
+        '''
+        try:
+            node.state = zk.DELETING
+            zk_conn.storeNode(node)
+            if node.external_id:
+                manager.cleanupNode(node.external_id)
+                manager.waitForNodeCleanup(node.external_id)
+        except exceptions.NotFound:
+            NodeDeleter.log.info("Instance %s not found in provider %s",
+                                 node.external_id, node.provider)
+        except Exception:
+            NodeDeleter.log.exception(
+                "Exception deleting instance %s from %s:",
+                node.external_id, node.provider)
+            # Don't delete the ZK node in this case, but do unlock it
+            if node_exists:
+                zk_conn.unlockNode(node)
+            return
+
+        if node_exists:
+            NodeDeleter.log.info(
+                "Deleting ZK node id=%s, state=%s, external_id=%s",
+                node.id, node.state, node.external_id)
+            # This also effectively releases the lock
+            zk_conn.deleteNode(node)
+            manager.nodeDeletedNotification(node)
+
+    def run(self):
+        # Since leaked instances won't have an actual node in ZooKeeper,
+        # we need to check 'id' to see if this is an artificial Node.
+        if self._node.id is None:
+            node_exists = False
+        else:
+            node_exists = True
+
+        try:
+            self.delete(self._zk, self._provider_manager,
+                        self._node, node_exists)
+        except Exception:
+            self.log.exception("Error deleting node %s:", self._node)
+
+
 class QuotaInformation:
 
     def __init__(self, cores=None, instances=None, ram=None, default=0):
@@ -117,11 +184,16 @@ class QuotaInformation:
         be initialized with default which will be typically 0 or math.inf
         indicating an infinite limit.
 
-        :param cores:
-        :param instances:
-        :param ram:
-        :param default:
+        :param cores: An integer number of (v)CPU cores.
+        :param instances: An integer number of instances.
+        :param ram: An integer amount of RAM in Mebibytes.
+        :param default: The default value to use for any attribute not supplied
+                        (usually 0 or math.inf).
         '''
+        # Note that the self.quota['compute'] map is inserted into ZK as
+        # a node property (via get_resources) and is consumed by Zuul to
+        # calculate resource usage.  Thus care should be taken if
+        # modifying fields below.
         self.quota = {
             'compute': {
                 'cores': self._get_default(cores, default),
@@ -173,6 +245,10 @@ class QuotaInformation:
                     return False
         return True
 
+    def get_resources(self):
+        '''Return resources value to register in ZK node'''
+        return self.quota['compute']
+
     def __str__(self):
         return str(self.quota)
 
@@ -235,7 +311,14 @@ class QuotaSupport:
 
         # This is initialized with the full tenant quota and later becomes
         # the quota available for nodepool.
-        nodepool_quota = self.getProviderLimits()
+        try:
+            nodepool_quota = self.getProviderLimits()
+        except Exception:
+            if self._current_nodepool_quota:
+                self.log.exception("Unable to get provider quota, "
+                                   "using cached value")
+                return copy.deepcopy(self._current_nodepool_quota['quota'])
+            raise
 
         self.log.debug("Provider quota for %s: %s",
                        self.provider.name, nodepool_quota)
@@ -291,3 +374,37 @@ class QuotaSupport:
                     self.log.exception("Couldn't consider invalid node %s "
                                        "for quota:" % node)
         return used_quota
+
+
+class RateLimiter:
+    """A Rate limiter
+
+    :param str name: The provider name; used in logging.
+    :param float rate_limit: The rate limit expressed in
+        requests per second.
+
+    Example:
+    .. code:: python
+
+        rate_limiter = RateLimiter('provider', 1.0)
+        with rate_limiter:
+            api_call()
+    """
+
+    def __init__(self, name, rate_limit):
+        self._running = True
+        self.name = name
+        self.delta = 1.0 / rate_limit
+        self.last_ts = None
+
+    def __enter__(self):
+        if self.last_ts is None:
+            return
+        while True:
+            delta = time.monotonic() - self.last_ts
+            if delta >= self.delta:
+                break
+            time.sleep(self.delta - delta)
+
+    def __exit__(self, etype, value, tb):
+        self.last_ts = time.monotonic()
