@@ -31,6 +31,7 @@ from nodepool import provider_manager
 from nodepool import stats
 from nodepool import config as nodepool_config
 from nodepool import zk
+from nodepool.driver.utils import QuotaInformation
 from nodepool.logconfig import get_annotated_logger
 
 
@@ -161,6 +162,20 @@ class PoolWorker(threading.Thread, stats.StatsReporter):
                                   req.provider, candidate_launchers)
                         continue
 
+            pm = self.getProviderManager()
+
+            # check tenant quota if the request has a tenant associated
+            # and there are resource limits configured for this tenant
+            check_tenant_quota = req.tenant_name and req.tenant_name \
+                in self.nodepool.config.tenant_resource_limits
+
+            if check_tenant_quota and not self._hasTenantQuota(req, pm):
+                # Defer request for it to be handled and fulfilled at a later
+                # run.
+                log.debug(
+                    "Deferring request because it would exceed tenant quota")
+                continue
+
             log.debug("Locking request")
             try:
                 self.zk.lockNodeRequest(req, blocking=False)
@@ -177,7 +192,6 @@ class PoolWorker(threading.Thread, stats.StatsReporter):
             # Got a lock, so assign it
             log.info("Assigning node request %s" % req)
 
-            pm = self.getProviderManager()
             rh = pm.getRequestHandler(self, req)
             rh.run()
             if rh.paused:
@@ -219,6 +233,52 @@ class PoolWorker(threading.Thread, stats.StatsReporter):
         active_reqs = [r.request.id for r in self.request_handlers]
         self.log.debug("Active requests: %s", active_reqs)
 
+    def _hasTenantQuota(self, request, provider_manager):
+        '''
+        Checks if a tenant has enough quota to handle a list of nodes.
+        This takes into account the all currently existing nodes as reported
+        by zk.
+
+        :param request: the node request in question
+        :param provider_manager: the provider manager for the request
+        :return: True if there is enough quota for the tenant, False otherwise
+        '''
+        log = get_annotated_logger(self.log, event_id=request.event_id,
+                                   node_request_id=request.id)
+
+        tenant_name = request.tenant_name
+        needed_quota = QuotaInformation()
+
+        pool = self.getPoolConfig()
+        for ntype in request.node_types:
+            # can not determine needed quota for ntype if label not in pool
+            # therefore just skip them here to avoid errors in
+            # 'quotaNeededByLabel'
+            if ntype not in pool.labels:
+                continue
+            needed_quota.add(provider_manager.quotaNeededByLabel(ntype, pool))
+
+        used_quota = self._getUsedQuotaForTenant(tenant_name)
+        tenant_quota = QuotaInformation(
+            default=math.inf,
+            **self.nodepool.config.tenant_resource_limits[tenant_name])
+
+        tenant_quota.subtract(used_quota)
+        log.debug("Current tenant quota: %s", tenant_quota)
+        tenant_quota.subtract(needed_quota)
+        log.debug("Predicted remaining tenant quota: %s", tenant_quota)
+        return tenant_quota.non_negative()
+
+    def _getUsedQuotaForTenant(self, tenant_name):
+        used_quota = QuotaInformation()
+        for node in self.zk.nodeIterator(cached_ids=True):
+            if not node.resources:
+                continue
+            if node.tenant_name == tenant_name:
+                resources = QuotaInformation(**node.resources)
+                used_quota.add(resources)
+        return used_quota
+
     # ---------------------------------------------------------------
     # Public methods
     # ---------------------------------------------------------------
@@ -256,27 +316,28 @@ class PoolWorker(threading.Thread, stats.StatsReporter):
         self.running = True
 
         while self.running:
-            # Don't do work if we've lost communication with the ZK cluster
-            did_suspend = False
-            while self.zk and (self.zk.suspended or self.zk.lost):
-                did_suspend = True
-                self.log.info("ZooKeeper suspended. Waiting")
-                time.sleep(SUSPEND_WAIT_TIME)
-            if did_suspend:
-                self.log.info("ZooKeeper available. Resuming")
-
-            # Make sure we're always registered with ZK
-            launcher = zk.Launcher()
-            launcher.id = self.launcher_id
-            for prov_cfg in self.nodepool.config.providers.values():
-                launcher.supported_labels.update(prov_cfg.getSupportedLabels())
-            launcher.provider_name = self.provider_name
-            self.zk.registerLauncher(launcher)
-
-            self.updateProviderLimits(
-                self.nodepool.config.providers.get(self.provider_name))
-
             try:
+                # Don't do work if we've lost communication with the ZK cluster
+                did_suspend = False
+                while self.zk and (self.zk.suspended or self.zk.lost):
+                    did_suspend = True
+                    self.log.info("ZooKeeper suspended. Waiting")
+                    time.sleep(SUSPEND_WAIT_TIME)
+                if did_suspend:
+                    self.log.info("ZooKeeper available. Resuming")
+
+                # Make sure we're always registered with ZK
+                launcher = zk.Launcher()
+                launcher.id = self.launcher_id
+                for prov_cfg in self.nodepool.config.providers.values():
+                    launcher.supported_labels.update(
+                        prov_cfg.getSupportedLabels())
+                launcher.provider_name = self.provider_name
+                self.zk.registerLauncher(launcher)
+
+                self.updateProviderLimits(
+                    self.nodepool.config.providers.get(self.provider_name))
+
                 if not self.paused_handler:
                     while not self._assignHandlers():
                         # _assignHandlers can take quite some time on a busy
@@ -1085,6 +1146,7 @@ class NodePool(threading.Thread):
                             t.start()
                             self._pool_threads[key] = t
                         elif not self._pool_threads[key].is_alive():
+                            self._pool_threads[key].stop()
                             self._pool_threads[key].join()
                             t = PoolWorker(self, provider.name, pool.name)
                             self.log.info("Restarting %s" % t.name)
