@@ -476,6 +476,7 @@ class NodeRequest(BaseModel):
         self.provider = None
         self.relative_priority = 0
         self.event_id = None
+        self.tenant_name = None
 
     def __repr__(self):
         d = self.toDict()
@@ -493,7 +494,8 @@ class NodeRequest(BaseModel):
                     self.requestor == other.requestor and
                     self.requestor_data == other.requestor_data and
                     self.provider == other.provider and
-                    self.relative_priority == other.relative_priority)
+                    self.relative_priority == other.relative_priority and
+                    self.tenant_name == other.tenant_name)
         else:
             return False
 
@@ -519,6 +521,7 @@ class NodeRequest(BaseModel):
         d['provider'] = self.provider
         d['relative_priority'] = self.relative_priority
         d['event_id'] = self.event_id
+        d['tenant_name'] = self.tenant_name
         return d
 
     @staticmethod
@@ -547,6 +550,7 @@ class NodeRequest(BaseModel):
         self.provider = d.get('provider')
         self.relative_priority = d.get('relative_priority', 0)
         self.event_id = d.get('event_id')
+        self.tenant_name = d.get('tenant_name')
 
 
 class Node(BaseModel):
@@ -581,6 +585,7 @@ class Node(BaseModel):
         self.external_id = None
         self.hostname = None
         self.comment = None
+        self.user_data = None
         self.hold_job = None
         self.username = None
         self.host_keys = []
@@ -588,6 +593,7 @@ class Node(BaseModel):
         self.resources = None
         self.attributes = None
         self.python_path = None
+        self.tenant_name = None
 
     def __repr__(self):
         d = self.toDict()
@@ -618,6 +624,7 @@ class Node(BaseModel):
                     self.external_id == other.external_id and
                     self.hostname == other.hostname and
                     self.comment == other.comment and
+                    self.user_data == other.user_data and
                     self.hold_job == other.hold_job and
                     self.username == other.username and
                     self.connection_type == other.connection_type and
@@ -627,7 +634,8 @@ class Node(BaseModel):
                     self.hold_expiration == other.hold_expiration and
                     self.resources == other.resources and
                     self.attributes == other.attributes and
-                    self.python_path == other.python_path)
+                    self.python_path == other.python_path and
+                    self.tenant_name == other.tenant_name)
         else:
             return False
 
@@ -668,6 +676,7 @@ class Node(BaseModel):
         d['external_id'] = self.external_id
         d['hostname'] = self.hostname
         d['comment'] = self.comment
+        d['user_data'] = self.user_data
         d['hold_job'] = self.hold_job
         d['host_keys'] = self.host_keys
         d['username'] = self.username
@@ -678,6 +687,7 @@ class Node(BaseModel):
         d['resources'] = self.resources
         d['attributes'] = self.attributes
         d['python_path'] = self.python_path
+        d['tenant_name'] = self.tenant_name
         return d
 
     @staticmethod
@@ -722,6 +732,7 @@ class Node(BaseModel):
         self.external_id = d.get('external_id')
         self.hostname = d.get('hostname')
         self.comment = d.get('comment')
+        self.user_data = d.get('user_data')
         self.hold_job = d.get('hold_job')
         self.username = d.get('username', 'zuul')
         self.connection_type = d.get('connection_type')
@@ -743,6 +754,7 @@ class Node(BaseModel):
         self.attributes = d.get('attributes')
         self.python_path = d.get('python_path')
         self.shell_type = d.get('shell_type')
+        self.tenant_name = d.get('tenant_name')
 
 
 class ZooKeeper(object):
@@ -2221,13 +2233,18 @@ class ZooKeeper(object):
 
         return False
 
-    def nodeIterator(self, cached=True):
+    def nodeIterator(self, cached=True, cached_ids=False):
         '''
         Utility generator method for iterating through all nodes.
 
         :param bool cached: True if the data should be taken from the cache.
+        :param bool cached_ids: True if the node IDs should be taken from the
+                                cache.
         '''
-        for node_id in self.getNodes():
+
+        node_ids = self._cached_nodes.keys() if cached_ids else self.getNodes()
+
+        for node_id in node_ids:
             node = self.getNode(node_id, cached=cached)
             if node:
                 yield node
